diff --git a/common/common.c b/common/common.c
index d7d45d3..2612021 100644
--- a/common/common.c
+++ b/common/common.c
@@ -630,18 +630,21 @@ static void x264_log_default( void *p_unused, int i_level, const char *psz_fmt,
 /****************************************************************************
  * x264_picture_alloc:
  ****************************************************************************/
-void x264_picture_alloc( x264_picture_t *pic, int i_csp, int i_width, int i_height )
+int x264_picture_alloc( x264_picture_t *pic, int i_csp, int i_width, int i_height )
 {
     pic->i_type = X264_TYPE_AUTO;
     pic->i_qpplus1 = 0;
     pic->img.i_csp = i_csp;
     pic->img.i_plane = 3;
     pic->img.plane[0] = x264_malloc( 3 * i_width * i_height / 2 );
+    if( !pic->img.plane[0] )
+        return -1;
     pic->img.plane[1] = pic->img.plane[0] + i_width * i_height;
     pic->img.plane[2] = pic->img.plane[1] + i_width * i_height / 4;
     pic->img.i_stride[0] = i_width;
     pic->img.i_stride[1] = i_width / 2;
     pic->img.i_stride[2] = i_width / 2;
+    return 0;
 }
 
 /****************************************************************************
@@ -710,10 +713,10 @@ void *x264_malloc( int i_size )
 #elif defined( HAVE_MALLOC_H )
     return memalign( 16, i_size );
 #else
-    uint8_t * buf;
-    uint8_t * align_buf;
-    buf = (uint8_t *) malloc( i_size + 15 + sizeof( void ** ) +
-              sizeof( int ) );
+    uint8_t *buf, *align_buf;
+    buf = malloc( i_size + 15 + sizeof( void ** ) + sizeof( int ) );
+    if( !buf )
+        return NULL;
     align_buf = buf + 15 + sizeof( void ** ) + sizeof( int );
     align_buf -= (intptr_t) align_buf & 15;
     *( (void **) ( align_buf - sizeof( void ** ) ) ) = buf;
@@ -753,7 +756,7 @@ void *x264_realloc( void *p, int i_size )
                          sizeof( int ) ) );
     }
     p_new = x264_malloc( i_size );
-    if( i_old_size > 0 && i_size > 0 )
+    if( i_old_size > 0 && i_size > 0 && p_new )
     {
         memcpy( p_new, p, ( i_old_size < i_size ) ? i_old_size : i_size );
     }
diff --git a/common/frame.c b/common/frame.c
index cc4b1b3..7745096 100644
--- a/common/frame.c
+++ b/common/frame.c
@@ -177,8 +177,10 @@ void x264_frame_delete( x264_frame_t *frame )
     x264_free( frame->mv[1] );
     x264_free( frame->ref[0] );
     x264_free( frame->ref[1] );
-    x264_pthread_mutex_destroy( &frame->mutex );
-    x264_pthread_cond_destroy( &frame->cv );
+    if( frame->mutex )
+        x264_pthread_mutex_destroy( &frame->mutex );
+    if( frame->cv )
+        x264_pthread_cond_destroy( &frame->cv );
     x264_free( frame );
 }
 
@@ -948,7 +950,8 @@ x264_frame_t *x264_frame_pop_unused( x264_t *h )
         frame = x264_frame_pop( h->frames.unused );
     else
         frame = x264_frame_new( h );
-    assert( frame->i_reference_count == 0 );
+    if( !frame )
+        return NULL;
     frame->i_reference_count = 1;
     frame->b_intra_calculated = 0;
     return frame;
diff --git a/encoder/encoder.c b/encoder/encoder.c
index 176443b..fdeae9d 100644
--- a/encoder/encoder.c
+++ b/encoder/encoder.c
@@ -618,7 +618,8 @@ static void mbcmp_init( x264_t *h )
  ****************************************************************************/
 x264_t *x264_encoder_open   ( x264_param_t *param )
 {
-    x264_t *h = x264_malloc( sizeof( x264_t ) );
+    x264_t *h;
+    CHECKED_MALLOC( h, sizeof( x264_t ) );
     char buf[1000], *p;
     int i;
 
@@ -768,14 +769,14 @@ x264_t *x264_encoder_open   ( x264_param_t *param )
     h->thread[0] = h;
     h->i_thread_num = 0;
     for( i = 1; i < h->param.i_threads; i++ )
-        h->thread[i] = x264_malloc( sizeof(x264_t) );
+        CHECKED_MALLOC( h->thread[i], sizeof(x264_t) );
 
     for( i = 0; i < h->param.i_threads; i++ )
     {
         if( i > 0 )
             *h->thread[i] = *h;
         h->thread[i]->fdec = x264_frame_pop_unused( h );
-        h->thread[i]->out.p_bitstream = x264_malloc( h->out.i_bitstream );
+        CHECKED_MALLOC( h->thread[i]->out.p_bitstream, h->out.i_bitstream );
         if( x264_macroblock_cache_init( h->thread[i] ) < 0 )
             return NULL;
     }
@@ -792,8 +793,7 @@ x264_t *x264_encoder_open   ( x264_param_t *param )
         else
         {
             x264_log( h, X264_LOG_ERROR, "can't write to fdec.yuv\n" );
-            x264_free( h );
-            return NULL;
+            goto fail;
         }
     }
 
@@ -804,6 +804,9 @@ x264_t *x264_encoder_open   ( x264_param_t *param )
         "High 4:4:4 Predictive", h->sps->i_level_idc/10, h->sps->i_level_idc%10 );
 
     return h;
+fail:
+    x264_free( h );
+    return NULL;
 }
 
 /****************************************************************************
diff --git a/encoder/ratecontrol.c b/encoder/ratecontrol.c
index 2dd34d0..2ad04ff 100644
--- a/encoder/ratecontrol.c
+++ b/encoder/ratecontrol.c
@@ -271,7 +271,8 @@ int x264_ratecontrol_new( x264_t *h )
 
     x264_emms();
 
-    rc = h->rc = x264_malloc( h->param.i_threads * sizeof(x264_ratecontrol_t) );
+    CHECKED_MALLOC( h->rc, h->param.i_threads * sizeof(x264_ratecontrol_t) );
+    rc = h->rc;
     memset( rc, 0, h->param.i_threads * sizeof(x264_ratecontrol_t) );
 
     rc->b_abr = h->param.rc.i_rc_method != X264_RC_CQP && !h->param.rc.b_stat_read;
@@ -373,8 +374,8 @@ int x264_ratecontrol_new( x264_t *h )
 
     rc->lstep = pow( 2, h->param.rc.i_qp_step / 6.0 );
     rc->last_qscale = qp2qscale(26);
-    rc->pred = x264_malloc( 5*sizeof(predictor_t) );
-    rc->pred_b_from_p = x264_malloc( sizeof(predictor_t) );
+    CHECKED_MALLOC( rc->pred, 5*sizeof(predictor_t) );
+    CHECKED_MALLOC( rc->pred_b_from_p, sizeof(predictor_t) );
     for( i = 0; i < 5; i++ )
     {
         rc->last_qscale_for[i] = qp2qscale( ABR_INIT_QP );
@@ -479,8 +480,8 @@ int x264_ratecontrol_new( x264_t *h )
             return -1;
         }
 
-        rc->entry = (ratecontrol_entry_t*) x264_malloc(rc->num_entries * sizeof(ratecontrol_entry_t));
-        memset(rc->entry, 0, rc->num_entries * sizeof(ratecontrol_entry_t));
+        CHECKED_MALLOC( rc->entry, rc->num_entries * sizeof(ratecontrol_entry_t) );
+        memset( rc->entry, 0, rc->num_entries * sizeof(ratecontrol_entry_t) );
 
         /* init all to skipped p frames */
         for(i=0; i<rc->num_entries; i++)
@@ -557,7 +558,7 @@ int x264_ratecontrol_new( x264_t *h )
     {
         char *p;
 
-        rc->psz_stat_file_tmpname = x264_malloc( strlen(h->param.rc.psz_stat_out) + 6 );
+        CHECKED_MALLOC( rc->psz_stat_file_tmpname, strlen(h->param.rc.psz_stat_out) + 6 );
         strcpy( rc->psz_stat_file_tmpname, h->param.rc.psz_stat_out );
         strcat( rc->psz_stat_file_tmpname, ".temp" );
 
@@ -585,6 +586,8 @@ int x264_ratecontrol_new( x264_t *h )
     }
 
     return 0;
+fail:
+    return -1;
 }
 
 static int parse_zone( x264_t *h, x264_zone_t *z, char *p )
@@ -607,7 +610,7 @@ static int parse_zone( x264_t *h, x264_zone_t *z, char *p )
     p += len;
     if( !*p )
         return 0;
-    z->param = x264_malloc( sizeof(x264_param_t) );
+    CHECKED_MALLOC( z->param, sizeof(x264_param_t) );
     memcpy( z->param, &h->param, sizeof(x264_param_t) );
     while( (tok = strtok_r( p, ",", &saveptr )) )
     {
@@ -625,6 +628,8 @@ static int parse_zone( x264_t *h, x264_zone_t *z, char *p )
         p = NULL;
     }
     return 0;
+fail:
+    return -1;
 }
 
 static int parse_zones( x264_t *h )
@@ -633,13 +638,13 @@ static int parse_zones( x264_t *h )
     int i;
     if( h->param.rc.psz_zones && !h->param.rc.i_zones )
     {
-        char *p, *tok, UNUSED *saveptr;
-        char *psz_zones = x264_malloc( strlen(h->param.rc.psz_zones)+1 );
+        char *psz_zones, *p, *tok, UNUSED *saveptr;
+        CHECKED_MALLOC( psz_zones, strlen(h->param.rc.psz_zones)+1 );
         strcpy( psz_zones, h->param.rc.psz_zones );
         h->param.rc.i_zones = 1;
         for( p = psz_zones; *p; p++ )
             h->param.rc.i_zones += (*p == '/');
-        h->param.rc.zones = x264_malloc( h->param.rc.i_zones * sizeof(x264_zone_t) );
+        CHECKED_MALLOC( h->param.rc.zones, h->param.rc.i_zones * sizeof(x264_zone_t) );
         p = psz_zones;
         for( i = 0; i < h->param.rc.i_zones; i++ )
         {
@@ -671,7 +676,7 @@ static int parse_zones( x264_t *h )
         }
 
         rc->i_zones = h->param.rc.i_zones + 1;
-        rc->zones = x264_malloc( rc->i_zones * sizeof(x264_zone_t) );
+        CHECKED_MALLOC( rc->zones, rc->i_zones * sizeof(x264_zone_t) );
         memcpy( rc->zones+1, h->param.rc.zones, (rc->i_zones-1) * sizeof(x264_zone_t) );
 
         // default zone to fall back to if none of the others match
@@ -679,7 +684,7 @@ static int parse_zones( x264_t *h )
         rc->zones[0].i_end = INT_MAX;
         rc->zones[0].b_force_qp = 0;
         rc->zones[0].f_bitrate_factor = 1;
-        rc->zones[0].param = x264_malloc( sizeof(x264_param_t) );
+        CHECKED_MALLOC( rc->zones[0].param, sizeof(x264_param_t) );
         memcpy( rc->zones[0].param, &h->param, sizeof(x264_param_t) );
         for( i = 1; i < rc->i_zones; i++ )
         {
@@ -689,6 +694,8 @@ static int parse_zones( x264_t *h )
     }
 
     return 0;
+fail:
+    return -1;
 }
 
 static x264_zone_t *get_zone( x264_t *h, int frame_num )
@@ -1704,18 +1711,18 @@ static double count_expected_bits( x264_t *h )
     return expected_bits;
 }
 
-static void vbv_pass2( x264_t *h )
+static int vbv_pass2( x264_t *h )
 {
     /* for each interval of buffer_full .. underflow, uniformly increase the qp of all
      * frames in the interval until either buffer is full at some intermediate frame or the
      * last frame in the interval no longer underflows.  Recompute intervals and repeat.
      * Then do the converse to put bits back into overflow areas until target size is met */
 
-    x264_ratecontrol_t *rcc = h->rc;
-    double *fills = x264_malloc((rcc->num_entries+1)*sizeof(double));
+    x264_ratecontrol_t *rcc = h->rc;   
     double all_available_bits = h->param.rc.i_bitrate * 1000. * rcc->num_entries / rcc->fps;
     double expected_bits = 0;
-    double adjustment;
+    double adjustment, *fills;
+    CHECKED_MALLOC( fills, (rcc->num_entries+1)*sizeof(double) );
     double prev_bits = 0;
     int i, t0, t1;
     double qscale_min = qp2qscale(h->param.rc.i_qp_min);
@@ -1763,6 +1770,9 @@ static void vbv_pass2( x264_t *h )
         rcc->entry[i].expected_vbv = rcc->buffer_size - fills[i];
 
     x264_free(fills-1);
+    return 0;
+fail:
+    return -1;
 }
 
 static int init_pass2( x264_t *h )
@@ -1914,7 +1924,8 @@ static int init_pass2( x264_t *h )
         x264_free(blurred_qscale);
 
     if(rcc->b_vbv)
-        vbv_pass2(h);
+        if(vbv_pass2(h) < 0)
+            return -1;
     expected_bits = count_expected_bits(h);
 
     if(fabs(expected_bits/all_available_bits - 1.0) > 0.01)
diff --git a/muxers.c b/muxers.c
index d62be5c..1f5fa60 100644
--- a/muxers.c
+++ b/muxers.c
@@ -440,7 +440,8 @@ typedef struct thread_input_arg_t {
 int open_file_thread( char *psz_filename, hnd_t *p_handle, x264_param_t *p_param )
 {
     thread_input_t *h = malloc(sizeof(thread_input_t));
-    x264_picture_alloc( &h->pic, X264_CSP_I420, p_param->i_width, p_param->i_height );
+    if( x264_picture_alloc( &h->pic, X264_CSP_I420, p_param->i_width, p_param->i_height ) < 0 )
+        return -1;
     h->p_read_frame = p_read_frame;
     h->p_close_infile = p_close_infile;
     h->p_handle = *p_handle;
diff --git a/x264.h b/x264.h
index 26ac421..6c4992c 100644
--- a/x264.h
+++ b/x264.h
@@ -358,7 +358,7 @@ typedef struct
 
 /* x264_picture_alloc:
  *  alloc data for a picture. You must call x264_picture_clean on it. */
-void x264_picture_alloc( x264_picture_t *pic, int i_csp, int i_width, int i_height );
+int x264_picture_alloc( x264_picture_t *pic, int i_csp, int i_width, int i_height );
 
 /* x264_picture_clean:
  *  free associated resource for a x264_picture_t allocated with